/*******************************************************************************
 shot-fa.js:

 $Author: dmc $
 $Revision: 3221 $
 $Date: 2023-11-16 18:06:33 -0400 (Thu, 16 Nov 2023) $
 $HeadURL: http://support2.silvermountaintargets.com/svn/smt/smt.common.lib/lib/messagehandler/shot-fa.js $
 $Id: shot-fa.js 3221 2023-11-16 22:06:33Z dmc $
*******************************************************************************/

/*eslint camelcase: "error"*/

var EventEmitter    = require( "events" ).EventEmitter;
var util            = require( "util" );
var log             = require( "log4js" ).getLogger( "ShotFAHandler" );
var Shot            = require( "../../models/shot.js" );

function ShotFAHandler() {

    this.numberOfDataBytes = 94; // 100 including the five 0xbb framing bytes plus the 0xaa id byte
    this.name = "ShotFA";

}

util.inherits( ShotFAHandler, EventEmitter );

/**
 * raw data of one shot is structured as followed:
 * 8 x counter values, while a single counter value contains:
 * byte0 -> number of channel
 * byte1 -> value high byte
 * byte2 -> value
 * byte3 -> value
 * byte4 -> value low byte
 * byte5 -> first stop condition byte
 * byte6 -> second stop condition byte
 */

 /*
bb bb bb bb bb aa // framing
offset  sample data
------  -----------
0       05 00 0005
4       051a // temp58
6       0a0a 0100000000 // MIK 1
13      0a0a 02,0002,00a3 // MIK 2
20      0a0a 03000200cd // MIK 3
27      0a0a 04000200f7 // MIK 4
34      0a0a 0500020121 // MIK 5
41      0a0a 060002014b // MIK 6
48      0a0a 0700020175 // MIK 7
55      0a0a 080002019f // MIK 8
62      0a0a 1a1a // temp58
66      0a0a ffff // temp47
70      0a0a ffff // temp26
74      0a0a ffff // temp13
78      0a0a 0000 0000 0000 0000 0000 0000 00xx // xx = circ buf size, v101+
------ total 94 bytes, plus 5 framing plus 1 id byte == 100 bytes


 */

ShotFAHandler.prototype.onMessage = function( data, idByte ) {

    var startOffset = 0;
    var buf = new Buffer( data );
    var shot = new Shot();
    var counterOffset = 8 + startOffset;
    var temperaturesOffsetSOLO = 62;
    var circBufOffsetSOLO = 93; // SW v101+, size of circular buffer

    // log.debug( "RAW: " + buf.toString( "hex" ) );
    // console.log( "RAW: ",Buffer([idByte]).toString("hex"),buf.toString( "hex" ),"len=",buf.length );

    // DMC 20180415 - omit shot.temperature; kluge handleIncomingShot() for now,
    // do it better when we refactor things

    shot.shotCountVector = buf[ 0 + startOffset ];
    shot.shotNumber = buf.readUInt16BE( 2 + startOffset );
    shot.circBufSize = buf[ circBufOffsetSOLO ] || 48;

    function get_timer_no(n) {
        var val = buf.readUInt32BE(1 + n * 7 + counterOffset);
        var highword = (val & 0xffff0000) >>> 16;
//console.log("DMC3 shot-fa #", n, (highword == 0xffff) ? ("overflow:"+val) : val,"val=",val.toString(16)/*hex*/);
//console.log("DMC3 shot-fa #", n,"val=",val.toString(16)/*hex*/, (highword == 0xffff) ? "overflow:" : "");
        if (highword == 0xffff) return -1;
        else return val;
    }
    shot.counter0 = get_timer_no(0);
    shot.counter1 = get_timer_no(1);
    shot.counter2 = get_timer_no(2);
    shot.counter3 = get_timer_no(3);
    shot.counter4 = get_timer_no(4);
    shot.counter5 = get_timer_no(5);
    shot.counter6 = get_timer_no(6);
    shot.counter7 = get_timer_no(7);

    if (shot.counter0 == -1 && shot.counter1 == -1 && shot.counter2 == -1 && shot.counter3 == -1
    && shot.counter4 == -1 && shot.counter5 == -1 && shot.counter6 == -1 && shot.counter7 == -1){
        console.log("PUBLIC FLOGGING -- ALL Overflow!!*****")
    }
    shot.SOLOtemperatures = {
        temperature58: this._calcTemperature(buf.readUInt16BE(2 + temperaturesOffsetSOLO + 0 * 4)),
        temperature47: this._calcTemperature(buf.readUInt16BE(2 + temperaturesOffsetSOLO + 1 * 4)),
        temperature26: this._calcTemperature(buf.readUInt16BE(2 + temperaturesOffsetSOLO + 2 * 4)),
        temperature13: this._calcTemperature(buf.readUInt16BE(2 + temperaturesOffsetSOLO + 3 * 4))
    }
    this._calcMeanTemperature(shot.SOLOtemperatures); // calc shot.SOLOtemperatures.mean_temp
    shot.mean_temp_uncal = shot.SOLOtemperatures.mean_temp;
    // we can probably delete shot.SOLOtemperatures

//    console.log( " shot temperatures--> " , shot.SOLOtemperatures );

    this.emit( "new-shot", shot );
};

ShotFAHandler.prototype._calcMeanTemperature = function (SOLOtemperatures) {
    var mean_temp = 0, n = 0;

    if ( 'undefined' != typeof SOLOtemperatures.temperature13 ) { n++; mean_temp += SOLOtemperatures.temperature13; }
    if ( 'undefined' != typeof SOLOtemperatures.temperature26 ) { n++; mean_temp += SOLOtemperatures.temperature26; }
    if ( 'undefined' != typeof SOLOtemperatures.temperature47 ) { n++; mean_temp += SOLOtemperatures.temperature47; }
    if ( 'undefined' != typeof SOLOtemperatures.temperature58 ) { n++; mean_temp += SOLOtemperatures.temperature58; }
    SOLOtemperatures.dbg_n_temp_rgs = n;
    if ( n ) mean_temp /= n; else mean_temp = undefined;
    SOLOtemperatures.mean_temp = mean_temp;
    return mean_temp;
}

ShotFAHandler.prototype._calcTemperature_legacy = function( adc ) { // defective
    var NTC_B = 3455; //from NTC Datasheet
    var NTC_Ts = 298.15;
    var NTC_Rs = 10000;
    var NTC_VRef = 3.3 / 1.6;
    var NTC_Vin = 3.3;
    var NTC_Res = 4095;
    var NTC_R1 = 68000; //see schematic
    var NTC_ADC = adc;
    var D1 = NTC_VRef / NTC_Res;
    var R2 = ( NTC_ADC * NTC_R1 * D1 ) / ( NTC_Vin - NTC_ADC * D1 );

    // check for missing or shorted sensor:
    if (adc < 3 || adc > 0x0fff-3 ) return undefined;

    // rounds to 2 decimal places
    return Math.round( 100.0 * ( ( ( NTC_B * NTC_Ts ) / ( NTC_B + NTC_Ts *  Math.log( R2 / NTC_Rs ) ) ) - 273.15 ) ) / 100.0;
};

// rework math and re-derive
ShotFAHandler.prototype._calcTemperature = function( adc_rdg/*0..4095 = 0..1600mV*/ ) {
    /*
        RN/R0 = exp(B*(1/T-1/T0))
        RN = thermistor resistance at T
        R0 = thermistor resistance at T0

        B*(1/T - 1/T0) = ln( R1/RN0 * v0/v1 / (1-v0/v1))
        --> T = B*T0 / (B + T0*(ln(R1/RN0 * v0/v1 /(1-v0/v1) )))

        R1 = pullup resistor to v1, R1-RN junction @ v0, RN to ground
        v0/v1 = 0..1
    */

    var NTC_B = 3380; // for T=25..50C
    var NTC_T0 = 298.15; // 25C
    var NTC_R0 = 10000; // resistance at 25C
    var NTC_R1 = 68000; // pullup

	// testcase: datasheet says R=12.081K at 20C -> reading=1274
    // testcase: R=10K at 25C --> reading 1083
	// testcase: datasheet says R=6.948K at 35C --> reading 783

    // var NTC_ADC = adc;
    // var D1 = NTC_VRef / NTC_Res;
    // var R2 = ( NTC_ADC * NTC_R1 * D1 ) / ( NTC_Vin - NTC_ADC * D1 );
    var vratscale = adc_rdg * 1.6/3.3 / 4095; // de-apply scaling... 1083--> 525

    var limit=100;
    // check for missing or shorted sensor:
    if (adc_rdg < limit || adc_rdg > 4095-limit ) return undefined;

    var temp_calcd = NTC_B*NTC_T0/(NTC_B + NTC_T0*Math.log(NTC_R1/NTC_R0*(vratscale)/(1-vratscale))) - 273.15;

    return Math.round(10*temp_calcd)/10 ; // round to 1 digit
};

ShotFAHandler.prototype._readUInt32 = function( values, offset ) {
  var result = 0;
  for ( var i = offset; i < offset + 4; i++ ) {
    result = ( result << 8 ) + values[ i ];
  }
  return result >>> 0;
};


module.exports = ShotFAHandler;

